package Praktikum3;

import java.util.*;

public class SnakeBot extends Bot {

    private Set<Integer> visited = new HashSet<>();

    public SnakeBot(String[] args) {
        super(args);
    }

    @Override
    protected char nextMove(View view) {
        if (view == null || view.data == null || view.data.isEmpty()) {
            System.err.println("FEHLER: View-Daten fehlen! Verbindung unterbrochen?");
            return 'q'; // Stoppt sicher
        }

        int width = view.width;
        String data = view.data;
        int pos = data.indexOf('A');

        if (pos == -1) {
            System.err.println("FEHLER: Rover nicht gefunden! Bewege mich zufällig.");
            return randomMove();
        }

        int rockPos = findClosestRock(pos, width, data);
        if (rockPos != -1) {
            return pathfindMove(pos, rockPos, width, data);
        }

        return exploreStrategically(pos, width, data);
    }

    private int findClosestRock(int pos, int width, String data) {
        Queue<Integer> queue = new LinkedList<>();
        Set<Integer> visited = new HashSet<>();
        queue.add(pos);
        visited.add(pos);

        while (!queue.isEmpty()) {
            int current = queue.poll();
            if (data.charAt(current) == '@') {
                return current; // `@` gefunden, direkt dorthin!
            }
            for (int next : getNeighbors(current, width, data)) {
                if (!visited.contains(next)) {
                    visited.add(next);
                    queue.add(next);
                }
            }
        }
        return -1; // Keine `@` gefunden
    }

    private List<Integer> getNeighbors(int pos, int width, String data) {
        List<Integer> neighbors = new ArrayList<>();
        int[] deltas = {-width, width, -1, 1};
        for (int delta : deltas) {
            int newPos = pos + delta;
            if (newPos >= 0 && newPos < data.length() && ".@".indexOf(data.charAt(newPos)) != -1) {
                neighbors.add(newPos);
            }
        }
        return neighbors;
    }

    private char pathfindMove(int pos, int target, int width, String data) {
        int x = pos % width, y = pos / width;
        int tx = target % width, ty = target / width;

        if (ty < y && canMove(pos - width, data)) return '^';
        if (ty > y && canMove(pos + width, data)) return 'v';
        if (tx < x && canMove(pos - 1, data)) return '<';
        if (tx > x && canMove(pos + 1, data)) return '>';

        return randomMove(); // Falls kein direkter Weg verfügbar ist, bewege dich zufällig
    }

    private char exploreStrategically(int pos, int width, String data) {
        List<Character> possibleMoves = new ArrayList<>();

        int[] deltas = {-width, width, -1, 1};
        char[] directions = {'^', 'v', '<', '>'};

        for (int i = 0; i < deltas.length; i++) {
            int newPos = pos + deltas[i];
            if (newPos >= 0 && newPos < data.length() && data.charAt(newPos) == '.') {
                if (!visited.contains(newPos)) {
                    visited.add(newPos);
                    return directions[i];
                }
                possibleMoves.add(directions[i]);
            }
        }

        if (!possibleMoves.isEmpty()) {
            return possibleMoves.get(new Random().nextInt(possibleMoves.size()));
        }

        return randomMove();
    }

    private boolean canMove(int pos, String data) {
        return pos >= 0 && pos < data.length() && ".@".indexOf(data.charAt(pos)) != -1;
    }

    private char randomMove() {
        char[] moves = {'^', 'v', '<', '>'};
        return moves[new Random().nextInt(moves.length)];
    }

    public static void main(String[] args) {
        try {
            new SnakeBot(args).run();
        } catch (Exception e) {
            System.err.println("Fehler im Bot-Hauptprozess: " + e.getMessage());
        }
    }
}





// docker run --rm -p 63187:63187 mediaeng/bots snakes